/****************************************************************************

    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#ifndef NEWT_LV2_VOICE_H_INCLUDED
#define NEWT_LV2_VOICE_H_INCLUDED

#include <lv2synth.hpp>
#include "Newtonator2.h"
#include "INewtSettings.h"
#include "newt_lv2_instr.peg"

class NewtonatorVoice : public LV2::Voice, public INewtSettings
{
    public:
    NewtonatorVoice();
    void on(unsigned char key, unsigned char velocity);
    void off(unsigned char velocity);
    unsigned char get_key() const;
    void render(uint32_t from, uint32_t to);
    inline Newtonator2& getNewt() { return _newt; }

    //INewtSettings
	stk::StkFloat getGravityScale(unsigned int chan) { return *p(p_grav_scale); }
	CeilingBehavEnum getCeilingBehavior(unsigned int chan) { return Newtonator2::GetCBFromFloat(*p(p_ceiling_behav)); }
	stk::StkFloat getVelocReset(unsigned int chan) { return *p(p_veloc_reset); }
	bool getStereo() { return (*p(p_stereo) == 1.0); }
	stk::StkFloat getVelocDelay(unsigned int chan) { return *p(p_veloc_delay); }
	stk::StkFloat getCeilingBounce(unsigned int chan) { return *p(p_ceil_bounce); }
	stk::StkFloat getDeadZone(unsigned int chan) { return *p(p_dead_zone); }
	#ifdef INCLUDE_EXT_INPUTS
	float* getFloorExtInput(unsigned int chan) { return p((chan == 0 ? p_floor_ext_input_lft : p_floor_ext_input_rgt)); }
	float* getGModExtInput(unsigned int chan) { return p((chan == 0 ? p_gmod_ext_input_lft : p_gmod_ext_input_rgt)); }
	#endif
	stk::StkFloat getWobbleA(unsigned int chan) { return *p(p_wobble_a); }
	stk::StkFloat getWobbleB(unsigned int chan) { return *p(p_wobble_b); }
	stk::StkFloat getWobbleScale(unsigned int chan) { return *p(p_wobble_scale); }
	stk::StkFloat getWobbleTimbre(unsigned int chan) { return *p(p_wobble_timbre); }
	stk::StkFloat getVelocRM(unsigned int chan) { return *p(p_veloc_rm); }
	stk::StkFloat getVelocSaH(unsigned int chan) { return *p(p_veloc_samphold); }
	stk::StkFloat getAmp(unsigned int chan) { return *p(p_amp); }

	//Amp Env settings
    stk::StkFloat getAmpAttack(unsigned int chan) { return *p(p_amp_attack); }
	stk::StkFloat getAmpDecay(unsigned int chan) { return *p(p_amp_decay); }
	stk::StkFloat getAmpSustain(unsigned int chan) { return *p(p_amp_sustain); }
	stk::StkFloat getAmpRelease(unsigned int chan) { return *p(p_amp_release); }

	//Gravity Modulator Settings
	stk::StkFloat getGModDepth(unsigned int chan) { return *p(p_gmod_depth); }
	stk::StkFloat getGModAttack(unsigned int chan) { return *p(p_gmod_attack); }
	stk::StkFloat getGModDecay(unsigned int chan) { return *p(p_gmod_decay); }
	stk::StkFloat getGModSustain(unsigned int chan) { return *p(p_gmod_sustain); }
	stk::StkFloat getGModRelease(unsigned int chan) { return *p(p_gmod_release); }
	stk::StkFloat getGModFrequency(unsigned int chan) { return *p(p_gmod_freq); }
	VariSource::Type getGModType(unsigned int chan) { return VariSource::getTypeFromFloat(*p(p_gmod_wav)); }
	bool getGModEnv(unsigned int chan) { return (*p(p_gmod_use_env) == 1.0); }
	stk::StkFloat getGModFreqDrift(unsigned int chan) { return *p(p_gmod_freq_drift); }
	stk::StkFloat getGModFreqDiv(unsigned int chan) { return *p(p_gmod_freq_div); }
	bool getGModFollowFreq(unsigned int chan) { return (*p(p_gmod_follow_freq) == 1.0); }
	stk::StkFloat getGModDCOffset(unsigned int chan) { return *p(p_gmod_dcoffset); }

    protected:
    unsigned char _key;
    Newtonator2 _newt;
    float _dummy;
};

#endif // NEWT_LV2_VOICE_H_INCLUDED
