/****************************************************************************

    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#include "applewidget.h"
#include "newt_scalar_widget.h"
#include <iostream>
#include <math.h>

using namespace std;

const double AppleWidget::m_defaultWidth = 23;
const double AppleWidget::m_defaultHeight = 23;

AppleWidget::AppleWidget(NewtScalarWidget& parent)
: m_parent(parent)
{
    //set_has_window(false);
    m_width = m_defaultWidth;
    m_height = m_defaultHeight;
    m_x = 0;
    m_y = 0;
    m_clickX = 0;
    m_clickY = 0;
    m_yBoundUpper = 0;
    m_yBoundLower = 0;
    m_screenHeight = (double)Gdk::Screen::get_default()->get_height();
    set_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::POINTER_MOTION_MASK);
}

AppleWidget::~AppleWidget()
{
    //dtor
}

void AppleWidget::on_map()
{
  //Call base class:
  Gtk::Widget::on_map();
}

void AppleWidget::on_unmap()
{
  //Call base class:
  Gtk::Widget::on_unmap();
}

Cairo::RefPtr<Cairo::Context> AppleWidget::InitNewContext()
{
    Cairo::RefPtr<Cairo::Context> ret;

    if (get_window())
    {
        ret = get_window()->create_cairo_context();
        ret->set_source_rgb(0.0, 0.0, 0.0);
        ret->set_line_width(2.0);
        //ret->set_antialias(Cairo::ANTIALIAS_GRAY);
    }

    return ret;
}

int AppleWidget::GetWidgetY(int y, int height)
{
    double ret = ((double)height / 2) - (m_height / 2) + y;

    return (int)ret;
}

int AppleWidget::GetWidgetX(int x, int width)
{
    //double ret = ((double)width / 2) - (m_width / 2) + x;

    //return (int)ret;
    return x;
}

void AppleWidget::MousePosChangeAbs(double x, double y)
{
    double range = m_parent.GetMax() - m_parent.GetMin();
    double pos = m_yBoundLower - y;

    if (pos > m_yBoundLower - m_yBoundUpper)
    {
        m_parent.SetValue(m_parent.GetMax());
    }
    else if (pos >= 0)
    {
        m_parent.SetValue(m_initVal + ((m_clickY - y) * (range / (m_yBoundLower - m_yBoundUpper))));
    }
    else
    {
        m_parent.SetValue(m_parent.GetMin());
    }
}

void AppleWidget::on_size_request(Gtk::Requisition* requisition)
{
    DrawingArea::on_size_request(requisition);

    //Initialize the output parameter:
    *requisition = Gtk::Requisition();

    //Discover the total amount of minimum space needed by this widget.

    //Let's make this simple example widget always need 23 by 23:
    requisition->height = m_defaultHeight;
    requisition->width = m_defaultWidth;

    //cout << requisition->width << "," << requisition->height << endl;
}

void AppleWidget::on_size_allocate(Gtk::Allocation& allocation)
{
    Glib::RefPtr<Gdk::Window> refGdkWindow = get_window();

    m_width = MIN(allocation.get_width(), allocation.get_height());
    m_height = m_width; //allocation.get_height();
    m_x = m_y = 0;
    double x = GetWidgetX(allocation.get_x(), allocation.get_width());
    double y = GetWidgetY(allocation.get_y(), allocation.get_height());

    //Use the offered allocation for this container:
    set_allocation(allocation);

    if(refGdkWindow)
    {
        refGdkWindow->move_resize( x, y, m_width, m_height );
    }

    //cout << "on_size_allocate: " << this << "," << m_x << "," << m_y << "," << m_width << "," << m_height << endl;
}

bool AppleWidget::on_expose_event(GdkEventExpose* event)
{
    if (get_window() && event)
    {
        Gtk::Allocation allocation = get_allocation();
        Cairo::RefPtr<Cairo::Context> context = InitNewContext();

        context->rectangle(0, 0, m_width, m_height);
        context->clip();

        DrawMe(context);
    }

    return true;
}

void AppleWidget::DrawMe(Cairo::RefPtr<Cairo::Context> context)
{
    double x = m_width;
    double y = m_height;
    double z = 0.03;
    double red = 1 - pow(m_parent.GetZeroToOneValue(), 2);
    double green = 1 - pow(m_parent.GetZeroToOneValue() - 1, 2);

    //cout << x << "," << y << endl;
    context->move_to(m_x+x*.333, m_y+y*1);
    context->line_to(m_x+x*.5, m_y+y*.9);
    context->line_to(m_x+x*.666, m_y+y*1);
    context->curve_to(m_x+x*1, m_y+y*.5, m_x+x*1, m_y+y*z, m_x+x*.75, m_y+y*z);
    context->curve_to(m_x+x*.5, m_y+y*z, m_x+x*.6, m_y+y*.1, m_x+x*.5, m_y+y*.1);
    context->curve_to(m_x+x*.4, m_y+y*.1, m_x+x*.5, m_y+y*z, m_x+x*.25, m_y+y*z);
    context->curve_to(m_x+x*z, m_y+y*z, m_x+x*z, m_y+y*.5, m_x+x*.333, m_y+y*1);

    context->stroke_preserve();

    //set foreground color based on value
    context->save();
    if (m_parent.get_sensitive())
        context->set_source_rgb(red, green, 0);
    else
        context->set_source_rgb(.7, .7, .7);
    context->fill();
    context->restore();

    //draw stem
    //context->move_to(m_x+x*.5, m_y+y*.1);
    //context->curve_to(m_x+x*.5, m_y+y*z, m_x+x*.5, m_y+y*.09, m_x+x*.53, m_y+y*z);
    //context->stroke();
}

void AppleWidget::on_spin_value_changed()
{
    queue_draw();
}

bool AppleWidget::on_button_press_event(GdkEventButton* event)
{
    double rng = m_screenHeight / 8;
    double valRatio = m_parent.GetZeroToOneValue();

    m_initVal = m_parent.GetValue();
    m_yBoundUpper = MAX(event->y_root - event->y - (rng * (1 - valRatio) * 2), 0); //upper as in screen pos, not pixel value
    m_yBoundLower = MIN(event->y_root - event->y + m_height + (rng * valRatio * 2), m_screenHeight); //lower as in screen pos, not pixel value
    m_clickX = event->x_root; // - event->x;
    m_clickY = event->y_root; // - event->y;

    //cout << "CLICK: " << m_yBoundUpper << "," << m_yBoundLower << "," << m_clickY << ", val = " << m_initVal << endl;

    switch(event->button)
    {
        case 1:
            add_modal_grab();
            MousePosChangeAbs(event->x_root, event->y_root);
            break;
        default:
            cout << "CLICK " << event->button << ": " << event->x_root << "," << event->y_root << "," << event->x << "," << event->y << endl;
            break;
    }

    return true;
}

bool AppleWidget::on_button_release_event(GdkEventButton* event)
{
    if (has_grab()) remove_modal_grab();

    return true;
}

bool AppleWidget::on_motion_notify_event(GdkEventMotion* event)
{
    if(has_grab()) MousePosChangeAbs(event->x_root, event->y_root);

    return true;
}
